searchNodes=[{"doc":"Caveat emptor: Protean is a library for personal learning and exploration, not (yet) for doing Serious Work . An experimental Elixir library for managing state and side-effects through the use of event-driven statecharts. It is heavily inspired by XState , a robust JavaScript/TypeScript statechart implementation, but strays to adhere to Elixir idioms and OTP conventions. Protean also attempts to follow the SCXML standard, though not completely. What are statecharts? They are an extension to finite state machines that allow you to model complex behavior in a declarative, data-driven manner. They include nested and parallel states, enhanced/augmented state (through context), side-effects (through actions), process management (through invoke), and more. To learn more about statecharts, I recommend statecharts.dev . Goals This project is currently an exploration of statecharts as they fit into the context of Elixir and OTP. XState adopted the actor model in its implementation, so Elixir seemed like a natural fit. However, it may be that Elixir/OTP makes these abstractions unnecessary. Example Add Protean.Supervisor under your application supervisor. This starts a supervisor that is used by Protean internally to manage subprocesses. children = [ Protean.Supervisor , # ... ] This simple statechart has a single state that defines the behavior of a counter with an optional maximum and minimum. defmodule Counter do use Protean alias Protean.Action defmachine ( initial : &quot;active&quot; , context : [ count : 0 , min : nil , max : nil ] , states : [ active : [ on : [ { &quot;Inc&quot; , actions : :increment , guard : [ not : :at_max ] } , { &quot;Dec&quot; , actions : :decrement , guard : [ not : :at_min ] } , { { &quot;Set&quot; , _ } , actions : :set_min_or_max } , { { &quot;Log&quot; , _ } , actions : :log } ] ] ] ) @impl Protean def action ( :increment , state , _event ) , do : Action . assign_in ( state , [ :count ] , &amp; &amp;1 + 1 ) def action ( :decrement , state , _event ) , do : Action . assign_in ( state , [ :count ] , &amp; &amp;1 - 1 ) def action ( :set_min_or_max , state , { &quot;Set&quot; , { key , val } } ) do state |&gt; Action . assign ( key , val ) end def action ( :log , state , { &quot;Log&quot; , attribute } ) do %{ context : context } = state IO . puts ( &quot; \#{ attribute } : \#{ context [ attribute ] } &quot; ) state end @impl Protean def guard ( :at_max , %{ context : %{ max : max , count : count } } , _event ) do max &amp;&amp; count &gt;= max end def guard ( :at_min , %{ context : %{ min : min , count : count } } , _event ) do min &amp;&amp; count &lt;= min end end It can be started under a supervisor, but we'll start it directly. { :ok , pid } = Protean . start_link ( Counter ) Protean . current ( pid ) . context # %{count: 0, min: nil, max: nil} Protean . send ( pid , &quot;Inc&quot; ) # :ok Enum . each ( 1 .. 4 , fn _ -&gt; Protean . send ( pid , &quot;Inc&quot; ) end ) Protean . current ( pid ) . context # %{count: 5, min: nil, max: nil} Protean . call ( pid , { &quot;Set&quot; , { :max , 10 } } ) # %Protean.State{ # context: %{count: 5, max: 10, min: nil}, # event: {&quot;Set&quot;, {:max, 10}}, # value: MapSet.new([[&quot;active&quot;, &quot;#&quot;]]) # } Enum . each ( 1 .. 20 , fn _ -&gt; Protean . send ( pid , &quot;Inc&quot; ) end ) Protean . send ( pid , { &quot;Log&quot; , :count } ) # count: 10 Defining a statechart Protean machines are event-driven statecharts , which means that, unlike ordinary finite-state machines, they can have complex, nested, potentially parallel states. This is more easily visualized than read, and I highly recommend looking at XState's introduction to state machines and statecharts for that reason. TODO: States TODO: Transitions TODO: Guards and automatic transitions TODO: Actions TODO: Invoked processes Starting supervised machines Just like GenServer , Protean machines will be most often started under a supervision tree. Invoking use Protean will automatically define a child_spec/1 function that allows you to start the process directly under a supervisor. children = [ Counter ] Supervisor . start_link ( children , strategy : :one_for_one ) Protean machines also accept the same options as Protean.start_link/3 . See those docs for more details. For instance, here's how you could start the Counter with a custom name: children = [ # Start the Counter machine { Counter , name : MyCounter } ] Supervisor . start_link ( children , strategy : :one_for_one ) Protean . current ( MyCounter ) # %Protean.State{ # context: %{count: 0, max: nil, min: nil}, # event: &quot;$protean.init&quot;, # value: MapSet.new([[&quot;active&quot;, &quot;#&quot;]]) # } Interacting with Protean machines Under the hood, a Protean machine is a GenServer , and Protean exposes a similar set of functions for interacting with one. You can see the individual docs for the functions in this module for details on their behavior, but here are some highlights. Familiar functions call/3 - Send an event synchronously to a Protean machine and receive the machine state resulting from any transitions in response. send/2 - Send an event asynchronously to a Protean machine. Always returns :ok . send_after/3 - Send an event to a Protean machine after a given delay. Like Process.send_after/4 , returns a timer reference so that the send can be canceled with Process.cancel_timer/2 . Additional functions specific to Protean machines current/1 - Get the current machine state of a running Protean machine. matches?/2 - Query the currently active state(s) of a machine. ask/3 - Like call/3 , but potentially returns an &quot;answer&quot; value in addition to the machine state. subscribe/2 (and unsubscribe/2 ) - Subscribes the calling process to receive a message on every state transition. Protean Supervisor Protean uses a DynamicSupervisor to manage internally spawned processes (often spawned through the use of :invoke ). The simplest thing to do is to add Protean.Supervisor in your application supervision tree: def start ( _type , _args ) do children = [ Protean.Supervisor , # ... ] Supervisor . start_link ( children , strategy : :one_for_one ) end This will start the supervisor under the name Protean.Supervisor and no additional configuration will be required. If you would like to start multiple supervisors, or a different type of supervisor (like a fancy PartitionSupervisor ), you can pass the new name as an option when starting a machine. Here's how that might look using the counter example from before. # in your supervision tree children = [ { Protean.Supervisor , name : ProteanSupervisor1 } , { Protean.Supervisor , name : ProteanSupervisor2 } ] # starting the counter Protean . start_link ( Counter , supervisor : ProteanSupervisor1 ) In the above example, any processes that are spawned by the Protean interpreter running Counter will use ProteanSupervisor1 .","ref":"Protean.html","title":"Protean","type":"behaviour"},{"doc":"Used to execute actions in response to machine transitions. Receives the current machine state and event triggering the action as arguments and must return the machine state. It is possible to attach actions to the machine state to indicate that they should be performed immediately following this action. See Protean.Action . Example defmachine ( # ... states : [ # ... state : [ on : [ { { :data , _any } , target : &quot;data_received&quot; , actions : &quot;assign_and_send_data&quot; } ] ] ] ) @impl Protean def action ( &quot;assign_and_send_data&quot; , state , { :data , data } ) do %{ service : pid } = state . context PubSub . broadcast! ( @pubsub , @topic , data ) state |&gt; Protean.Action . send ( { &quot;data_received&quot; , data } , to : pid ) |&gt; Protean.Action . assign ( :last_received_data , data ) end","ref":"Protean.html#c:action/3","title":"Protean.action/3","type":"callback"},{"doc":"Makes a synchronous call to the machine and waits for it to execute any transitions that result from the given event, returning an answer and the machine state. Behaves like ask/3 , but raises if an answer is not returned.","ref":"Protean.html#ask!/2","title":"Protean.ask!/2","type":"function"},{"doc":"","ref":"Protean.html#ask!/3","title":"Protean.ask!/3","type":"function"},{"doc":"Makes a synchronous call to the machine and waits for it to execute any transitions that result from the given event, returning a possible answer and the new machine state. Returns one of: {{:ok, answer}, state} - Returned if any actions executed as a result of the event set an answer through the use of Action.answer/2 . {nil, state} - Returned if no actions execute or if no executed actions set an answer. Answers are only returned to the caller if they result from the given event. If an asynchronous call, through send/2 for example, would have resulted in an answer, it will be &quot;lost&quot;.","ref":"Protean.html#ask/2","title":"Protean.ask/2","type":"function"},{"doc":"See Protean.Interpreter.Server.ask/3 .","ref":"Protean.html#ask/3","title":"Protean.ask/3","type":"function"},{"doc":"Makes a synchronous call to the machine and waits for it to execute any transitions that result from the given event, returning the new machine state. Shares semantics with GenServer.call/3 . See those docs for timeout behavior.","ref":"Protean.html#call/2","title":"Protean.call/2","type":"function"},{"doc":"See Protean.Interpreter.Server.call/3 .","ref":"Protean.html#call/3","title":"Protean.call/3","type":"function"},{"doc":"Synchronously retrieve the current machine state. TODO: Allow optional timeout as with call/3 .","ref":"Protean.html#current/1","title":"Protean.current/1","type":"function"},{"doc":"Used to determine whether a transition should take place. Example defmachine ( # ... states : [ editing_user : [ on : [ { { :user_commit , _ } , guard : &quot;user_valid?&quot; , actions : [ &quot;broadcast&quot; ] , target : &quot;viewing_user&quot; } , { { :user_commit , _ } , guard : { :not , &quot;user_valid?&quot; } , actions : [ &quot;show_invalid_user_error&quot; ] } ] ] ] ) @impl Protean def guard ( &quot;user_valid?&quot; , state , { _ , user } ) do User . changeset ( % User { } , user ) . valid? end","ref":"Protean.html#c:guard/3","title":"Protean.guard/3","type":"callback"},{"doc":"Used to define invoked services at runtime. Returns a value or child spec usable by the invoke type. Example defmachine ( # ... states : [ # ... awaiting_task : [ invoke : [ task : &quot;my_task&quot; , done : &quot;task_complete&quot; ] ] , task_complete : [ # ... ] ] ) @impl Protean def invoke ( &quot;my_task&quot; , _state , { &quot;trigger&quot; , data } ) do { __MODULE__ , :run_task , [ data ] } end","ref":"Protean.html#c:invoke/3","title":"Protean.invoke/3","type":"callback"},{"doc":"Returns true if the machine is currently in the given state. Note that calling matches?/2 on a machine process is a synchronous operation that is equivalent to: machine |&gt; Protean . current ( ) |&gt; Protean . matches? ( descriptor )","ref":"Protean.html#matches?/2","title":"Protean.matches?/2","type":"function"},{"doc":"Sends an asynchronous event to the machine. Shares semantics with GenServer.cast/2 .","ref":"Protean.html#send/2","title":"Protean.send/2","type":"function"},{"doc":"Sends an event to the machine after time in milliseconds has passed. Returns a timer reference that can be canceled with Process.cancel_timer/1 .","ref":"Protean.html#send_after/3","title":"Protean.send_after/3","type":"function"},{"doc":"Start a Protean machine linked to the current process. This is often used to start the machine as part of a supervision tree. See GenServer.start_link/3 for description of return value. The semantics are similar to GenServer.start_link/3 and accepts the same options, with the addition of some specific to Protean. Options :machine - defaults to module.machine() - %Protean.Machine{} that will be executed by the Protean interpreter. :handler - defaults to module - callback module used for actions, guards, invoke, etc. See &quot;Callbacks&quot;. :parent - defaults to self() - process id of the parent that will receive events from the machine if a Protean.Action.send(..., to: :parent) action is used or when the machine reaches a state with :type of :final . :supervisor - defaults to Protean.Supervisor - name of the supervisor process that will be used to start processes resulting from running the machine. See &quot;Supervisor&quot;. Any option accepted by GenServer.start_link/3 .","ref":"Protean.html#start_link/2","title":"Protean.start_link/2","type":"function"},{"doc":"See Protean.Interpreter.Server.stop/1 .","ref":"Protean.html#stop/1","title":"Protean.stop/1","type":"function"},{"doc":"TODO","ref":"Protean.html#stop/2","title":"Protean.stop/2","type":"function"},{"doc":"Subscribes the caller to a running machine, returning a reference. Processes subscribed to a machine will receive messages whenever the machine transitions. (Note that a machine can transition to the same state it was in previously.) By default, subscribed processes also monitor the machine (see Process.monitor/1 ). This behavior can be changed by passing monitor: false . Messages on transition will be delivered in the shape of: { :state , state , ref } where: state is the Protean.State resulting from the transition; ref is a monitor reference. As with monitor, if the process is already dead when calling Protean.subscribe/2 , a :DOWN message is delivered immediately.","ref":"Protean.html#subscribe/2","title":"Protean.subscribe/2","type":"function"},{"doc":"Unsubscribes the caller from the machine.","ref":"Protean.html#unsubscribe/2","title":"Protean.unsubscribe/2","type":"function"},{"doc":"Any message sent to a Protean machine.","ref":"Protean.html#t:event/0","title":"Protean.event/0","type":"type"},{"doc":"Option values for start* functions.","ref":"Protean.html#t:option/0","title":"Protean.option/0","type":"type"},{"doc":"A running Protean machine process.","ref":"Protean.html#t:server/0","title":"Protean.server/0","type":"type"},{"doc":"Options for subscribe/2 .","ref":"Protean.html#t:subscribe_option/0","title":"Protean.subscribe_option/0","type":"type"},{"doc":"Protean manages state, processes, and side-effects through actions , data structures describing things that should occur as a result of a transition. When a Protean machine transitions from one state to the next (or even &quot;self-transitions&quot; back to the same state), the interpreter collects any actions that should be performed as a result of that transition. Actions are then executed in a specific order: Exit actions - Any :exit actions specified on states that are exiting as a result of the transition. Transition actions - Any :actions specified on the transition that responded to the event. Entry actions - Any :entry actions specified on the states being entered. The execution of an action can change the state of the machine (it's context, for example), or run some side-effect (like broadcasting a PubSub message). Action execution can additionally produce more actions that will be executed immediately before moving onto the next top-level action as described above. High-level API The most common way to use actions is through the Protean.action/3 callback. This callback is run when an action is specified like this: [ # ... on : [ { :some_event , actions : [ :first_action , :second_action ] } ] ] These are then handled in callbacks: @impl Protean def action ( :first_action , state , _event ) do # ... state end def action ( :second_action , state , _event ) do # ... state end Action callbacks must always return the machine state, but they can attach actions to that state that will be immediately executed by the interpreter. For instance: def action ( :update_data , state , { :data_updated , changes } ) do state |&gt; Action . assign_in ( [ :data ] , &amp; Map . merge ( &amp;1 , changes ) ) end In this case, assign_in/3 is being used to update some data in the machine :context . But, we could perform additional actions if we wish, such as: def action ( :update_data , state , { :data_updated , changes } ) do %{ topic : topic , other_process : pid , data : data } = state . context new_data = Map . merge ( data , changes ) PubSub . broadcast! ( @pubsub , topic , { :data_updated , changes } ) state |&gt; Action . send ( { :data_commited , new_data } , to : pid ) |&gt; Action . assign ( :data , new_data ) end Note: Best practice is to let the interpreter execute as many actions as possible, as opposed to performing explicit side-effects in the callback. This is because actions can potentially halt the execution of any remaining actions, but side-effects executed in the callback will occur before any of the returned actions can be executed. Protean.Action provides a number of helpers for specifying actions. See individual function documentation for more. Low-level API: the action behaviour Ultimately, all actions are resolving to t:action() , a two-element tuple of a module and an argument. The module is expected to implement the Protean.Action behaviour (see &quot;Callbacks&quot;). Many of Protean's more &quot;dynamic&quot; features boil down to syntax sugar over actions, including :invoke and delayed transitions using :after . Modules implementing the action behaviour have direct access to the interpreter and therefore must be careful, well, not to muck anything up.","ref":"Protean.Action.html","title":"Protean.Action","type":"behaviour"},{"doc":"","ref":"Protean.Action.html#answer/1","title":"Protean.Action.answer/1","type":"function"},{"doc":"TODO","ref":"Protean.Action.html#answer/2","title":"Protean.Action.answer/2","type":"function"},{"doc":"","ref":"Protean.Action.html#assign/1","title":"Protean.Action.assign/1","type":"function"},{"doc":"","ref":"Protean.Action.html#assign/2","title":"Protean.Action.assign/2","type":"function"},{"doc":"TODO","ref":"Protean.Action.html#assign/3","title":"Protean.Action.assign/3","type":"function"},{"doc":"","ref":"Protean.Action.html#assign_in/2","title":"Protean.Action.assign_in/2","type":"function"},{"doc":"TODO","ref":"Protean.Action.html#assign_in/3","title":"Protean.Action.assign_in/3","type":"function"},{"doc":"","ref":"Protean.Action.html#choose/1","title":"Protean.Action.choose/1","type":"function"},{"doc":"TODO","ref":"Protean.Action.html#choose/2","title":"Protean.Action.choose/2","type":"function"},{"doc":"","ref":"Protean.Action.html#delegate/1","title":"Protean.Action.delegate/1","type":"function"},{"doc":"TODO","ref":"Protean.Action.html#delegate/2","title":"Protean.Action.delegate/2","type":"function"},{"doc":"Executes an action given an interpreter. See exec_action/2 .","ref":"Protean.Action.html#exec/2","title":"Protean.Action.exec/2","type":"function"},{"doc":"","ref":"Protean.Action.html#exec_action/2","title":"Protean.Action.exec_action/2","type":"function"},{"doc":"Accepts the action_arg passed with the action as well as the Protean interpreter. Returns one of three values: {:cont, interpreter} to continue running the action pipeline {:cont, interpreter, [action]} to inject actions that will be run immediately before running the rest of the pipeline's actions {:halt, interpreter} to halt the action pipeline, canceling any further actions","ref":"Protean.Action.html#c:exec_action/2","title":"Protean.Action.exec_action/2","type":"callback"},{"doc":"","ref":"Protean.Action.html#invoke/2","title":"Protean.Action.invoke/2","type":"function"},{"doc":"TODO","ref":"Protean.Action.html#put/2","title":"Protean.Action.put/2","type":"function"},{"doc":"","ref":"Protean.Action.html#send/2","title":"Protean.Action.send/2","type":"function"},{"doc":"TODO","ref":"Protean.Action.html#send/3","title":"Protean.Action.send/3","type":"function"},{"doc":"An action is a 2-element tuple, where handler implements the Protean.Action behaviour and action_arg is an argument that will be passed back to the handler during execution.","ref":"Protean.Action.html#t:action/0","title":"Protean.Action.action/0","type":"type"},{"doc":"","ref":"Protean.Action.html#t:exec_action_return/0","title":"Protean.Action.exec_action_return/0","type":"type"},{"doc":"","ref":"Protean.Events.Platform.html","title":"Protean.Events.Platform","type":"module"},{"doc":"Protocol for guarded transitions, actions, etc. Default implementations are provided for: BitString - Call handler module with string, state, and event Function - Call function with state and event Tuple - Higher-order guard utilities: {:and, [guard1, ...]} {:or, [guard1, ...]} {:not, guard} {:in, query} - Delegates to Protean.State.matches?/2","ref":"Protean.Guard.html","title":"Protean.Guard","type":"protocol"},{"doc":"","ref":"Protean.Guard.html#allows?/4","title":"Protean.Guard.allows?/4","type":"function"},{"doc":"Implements Protean.Guard protocol.","ref":"Protean.Guard.html#t:guard/0","title":"Protean.Guard.guard/0","type":"type"},{"doc":"","ref":"Protean.Guard.html#t:t/0","title":"Protean.Guard.t/0","type":"type"},{"doc":"Execution logic for a Protean machine.","ref":"Protean.Interpreter.html","title":"Protean.Interpreter","type":"module"},{"doc":"Handle an event, executing any transitions, actions, and side-effects associated with the current machine state. Returns a possible answer and the interpreter state: {{:ok, answer}, interpreter} - if any actions resulting from handling the event set an answer on the machine state; {nil, interpreter} - if no actions ran or none that ran set an answer.","ref":"Protean.Interpreter.html#handle_event/2","title":"Protean.Interpreter.handle_event/2","type":"function"},{"doc":"Create a new Interpreter . The returned interpreter will still need to be started, which could result in additional side-effects. See start/1 .","ref":"Protean.Interpreter.html#new/1","title":"Protean.Interpreter.new/1","type":"function"},{"doc":"Whether the interpreter has been started and can accept events.","ref":"Protean.Interpreter.html#running?/1","title":"Protean.Interpreter.running?/1","type":"function"},{"doc":"Entrypoint for the interpreter that must be called before the interpreter will be in a state where it can handle external events. This is necessary in order to handle any initializing actions, invokes, or automatic transitions. Calling start/1 on an already-running interpreter is a no-op.","ref":"Protean.Interpreter.html#start/1","title":"Protean.Interpreter.start/1","type":"function"},{"doc":"Return the current machine state.","ref":"Protean.Interpreter.html#state/1","title":"Protean.Interpreter.state/1","type":"function"},{"doc":"Stop an interpreter, preventing further event processing and terminating any invoked processes.","ref":"Protean.Interpreter.html#stop/1","title":"Protean.Interpreter.stop/1","type":"function"},{"doc":"","ref":"Protean.Interpreter.html#subscribe/2","title":"Protean.Interpreter.subscribe/2","type":"function"},{"doc":"","ref":"Protean.Interpreter.html#unsubscribe/2","title":"Protean.Interpreter.unsubscribe/2","type":"function"},{"doc":"Sets the context of the current state.","ref":"Protean.Interpreter.html#with_context/2","title":"Protean.Interpreter.with_context/2","type":"function"},{"doc":"","ref":"Protean.Interpreter.html#t:invoked/0","title":"Protean.Interpreter.invoked/0","type":"type"},{"doc":"","ref":"Protean.Interpreter.html#t:invoked_id/0","title":"Protean.Interpreter.invoked_id/0","type":"type"},{"doc":"","ref":"Protean.Interpreter.html#t:invoked_service/0","title":"Protean.Interpreter.invoked_service/0","type":"opaque"},{"doc":"","ref":"Protean.Interpreter.html#t:option/0","title":"Protean.Interpreter.option/0","type":"type"},{"doc":"","ref":"Protean.Interpreter.html#t:options/0","title":"Protean.Interpreter.options/0","type":"type"},{"doc":"","ref":"Protean.Interpreter.html#t:t/0","title":"Protean.Interpreter.t/0","type":"type"},{"doc":"Purely-functional statechart core. Provides the underlying state-transition logic for a statechart, primarily through transition/3 , a higher-level transition API for using %Protean.Machine{} apart from the interpreter provided by Protean, and take_transitions/3 , a lower-level API used by a statechart interpreter. It is uncommon to use this module independently of the Protean behaviour.","ref":"Protean.Machine.html","title":"Protean.Machine","type":"module"},{"doc":"Returns the initial Protean.State for a given machine.","ref":"Protean.Machine.html#initial_state/1","title":"Protean.Machine.initial_state/1","type":"function"},{"doc":"","ref":"Protean.Machine.html#new/2","title":"Protean.Machine.new/2","type":"function"},{"doc":"","ref":"Protean.Machine.html#select_automatic_transitions/2","title":"Protean.Machine.select_automatic_transitions/2","type":"function"},{"doc":"","ref":"Protean.Machine.html#select_transitions/3","title":"Protean.Machine.select_transitions/3","type":"function"},{"doc":"","ref":"Protean.Machine.html#take_transitions/3","title":"Protean.Machine.take_transitions/3","type":"function"},{"doc":"Given a machine, a machine state, and an event, transition to the next state if the machine defines a transition for the given state and event.","ref":"Protean.Machine.html#transition/3","title":"Protean.Machine.transition/3","type":"function"},{"doc":"A full Protean machine configuration.","ref":"Protean.Machine.html#t:t/0","title":"Protean.Machine.t/0","type":"type"},{"doc":"This module provides utilities to convert from the &quot;shorthand&quot; machine config format used when defining a machine to the structured format used internally.","ref":"Protean.MachineConfig.html","title":"Protean.MachineConfig","type":"module"},{"doc":"Parses machine config into a Node .","ref":"Protean.MachineConfig.html#parse!/1","title":"Protean.MachineConfig.parse!/1","type":"function"},{"doc":"Injects code into the calling module in order to easily define Protean statecharts. This module is included by calling use Protean and requires that the caller define a machine configuration using defmachine/1 : defmodule Example do use Protean defmachine [ # ... ] end","ref":"Protean.Macros.html","title":"Protean.Macros","type":"module"},{"doc":"Define a Protean machine accessible through __MODULE__.machine/0 .","ref":"Protean.Macros.html#defmachine/1","title":"Protean.Macros.defmachine/1","type":"macro"},{"doc":"The Protean machine state. A %Protean.State{} struct encompasses the current state of a machine, including its state configuration (value), the event that triggered a transition to this state, the current context (extended state), and other private state used internally by Protean. Additionally, the action callback c:pure/3 receives and returns state to update the context and schedule side-effects. See Protean.Action .","ref":"Protean.State.html","title":"Protean.State","type":"module"},{"doc":"","ref":"Protean.State.html#assign/2","title":"Protean.State.assign/2","type":"function"},{"doc":"Assign data to a state's context. Usage: assign(state, key, value) - Assigns value to key in state's context. assign(state, %{}) - Merges the update map into a state's context. assign(state, enumerable) - Collects the key/values of enumerable into a map, then merges that map into the state's context.","ref":"Protean.State.html#assign/3","title":"Protean.State.assign/3","type":"function"},{"doc":"TODO: Descriptor usage","ref":"Protean.State.html#matches?/2","title":"Protean.State.matches?/2","type":"function"},{"doc":"","ref":"Protean.State.html#t:context/0","title":"Protean.State.context/0","type":"type"},{"doc":"","ref":"Protean.State.html#t:private_state/0","title":"Protean.State.private_state/0","type":"opaque"},{"doc":"","ref":"Protean.State.html#t:t/0","title":"Protean.State.t/0","type":"type"},{"doc":"","ref":"Protean.State.html#t:value/0","title":"Protean.State.value/0","type":"type"},{"doc":"TODO Event descriptors Guards Internal Exact Actions","ref":"Protean.Transition.html","title":"Protean.Transition","type":"module"},{"doc":"Checks whether the transition is enabled for the given event.","ref":"Protean.Transition.html#enabled?/4","title":"Protean.Transition.enabled?/4","type":"function"},{"doc":"","ref":"Protean.Transition.html#t:t/0","title":"Protean.Transition.t/0","type":"type"},{"doc":"","ref":"Protean.Macros.ConfigError.html","title":"Protean.Macros.ConfigError","type":"exception"}]